package epsilonStandalone

import java.io.File
import java.io.FileWriter
import org.eclipse.emf.common.util.URI
import org.eclipse.emf.ecore.plugin.EcorePlugin
import org.eclipse.epsilon.emc.emf.EmfModel

class DNIRunner_Demo  {
	
	public def static void main(String[] args) {
	  	
	  	// important for storing files
	  	var dir = "/Users/Piotr/WuePhD/Research/CleanDNI/DNI-standalone/src/epsilonStandalone/"
		EcorePlugin.getPlatformResourceMap().put("models", URI.createFileURI(dir+"models/") );
	  	
	  	// STEP 1: check input model
	  	var EmfModel inputModel = EpsilonBase.readEmfModel("DNI", "models/scenario0A.model", "metamodels/dnimm3.ecore");
	  	var EmfModel dni1 = EpsilonBase.applyEolScript(inputModel,"eol/DNI-model-check.eol", newLinkedList())
//	  	dni1.store("models/scenario0A-dni1.model") // OK
	  	
	  	// STEP 2: generate required SDN Flow routes
	  	var EmfModel dni2 = EpsilonBase.applyEolScript(dni1,"eol/DNI-fix-sdn-routing.eol", newLinkedList())
//	  	dni2.store("models/scenario0A-dni2.model") // OK
	  	
	  	// STEP 3: auto-name
	  	var EmfModel dni3 = EpsilonBase.applyEolScript(dni2,"eol/DNI-auto-name.eol", newLinkedList())
	  	dni3.store("models/scenario0A-dni3.model") // OK	

	  	// STEP 4: transform flow-routing to classical routing
	  	var EmfModel dni4 = EpsilonBase.applyEolScript(dni2,"eol/DNI-routing-DNI.eol", newLinkedList())
	  	dni4.store("models/scenario0A-dni4.model") // OK	
	  	
		// STEP 5: transform to QPN
	  	var qpnOutputURI = "models/scenario0A-qpn.model"
		var qpn = EpsilonBase.createEmfModelByURI("QPME", qpnOutputURI, "metamodels/qpme.ecore", false, true);
	  	EpsilonBase.applyEtlScript(dni4, qpn, "etl/DNI-QPN.etl", newLinkedList())
		qpn.store(qpnOutputURI)

		// STEP 6: run model-to-text
		var text = EpsilonBase.applyEglScript(qpn, "egl/dni2qpme.egl", newLinkedList())
		
		
		
		var file = saveOutput(text, dir+qpnOutputURI+".qpe")
		println(file)
  	}
  	
	public static def saveOutput(String txt, String fname){
		var File file = new File(fname);
      	file.createNewFile();
      	var FileWriter writer = new FileWriter(file); 
      	writer.write(txt); 
      	writer.flush();
      	writer.close();
      	return file;
	}
 
}

