package dni3toOmnetTransformation

import OMNET.NedFile
import OMNET.OMNETPackage
import OMNET.impl.OMNETPackageImpl
import OMNET.OMNETFactory
import OMNET.impl.OMNETFactoryImpl
import dnimm3.Common
import dnimm3.Dnimm3Package
import dnimm3.NamedElement
import dnimm3.NetworkInfrastructure
import dnimm3.Node
import java.io.FileOutputStream
import org.eclipse.emf.common.util.URI
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl
import OMNET.Network
import OMNET.SubModule
import OMNET.impl.SubModuleImpl
import OMNET.SimpleModule
import OMNET.impl.SimpleModuleImpl
import OMNET.LocalType
import OMNET.impl.LocalTypeImpl
import OMNET.impl.NetworkImpl
import OMNET.Channel
import OMNET.Direction
import OMNET.Connection
import OMNET.ConnectionGroup
import OMNET.Connections
import dnimm3.SDN
import OMNET.InterfaceType
import dnimm3.NetworkInterface
import OMNET.CompoundModule
import dnimm3.CommunicatingApplication
import dnimm3.Flow
import javax.swing.text.FlowView.FlowStrategy
import OMNET.OmnetppIniGroup
import dnimm3.End

class Main {
	val static NAME = "dni3toomnet"
	
	val static fac = OMNETFactory.eINSTANCE
	val static omn = OMNETPackage.eINSTANCE
	
	//extension OMNETFactory = OMNETFactory.eINSTANCE
	//extension OMNETPackage = OMNETFactory.OMNETPackage

	public def static void main(String[] args) {
		
		var fname = "minimal.dnimm3"
		if (args.length < 1) {
			println(NAME +  " expects a filename as its first input, see --help for more information. Continuing with default file minimal.dnimm3")
		} else if (args.get(0).equals("--help")) {
			print("TODO Help")
			return
		}
		else{
			fname = args.get(0)
		}

		//TODO change DeterministicFunction to UpperCase, change StoppingRules and SimulationScenario
		val dnimm3Uri = URI.createFileURI(fname)
		val omnetUri = URI.createFileURI("omnet_base.omnetmodel") //this is some existing example - ideally should be empty file
		val resourceSet = new ResourceSetImpl()

        // register QPME Model
        val packageRegistry = resourceSet.getPackageRegistry()
        //OMNETPackageImpl.initializePackageContents()
        packageRegistry.put(OMNETPackage.eNS_URI, OMNETPackage.eINSTANCE)
        packageRegistry.put(Dnimm3Package.eNS_URI, Dnimm3Package.eINSTANCE) //dnimm
        
        // Globally register XML resource as UMLResource.Factory.Instance
        val extensionFactoryMap = Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
        extensionFactoryMap.put("dnimm3", new XMIResourceFactoryImpl())  //we use standard factories for XMI. This means a file with extension .dnimm3 is an xmi
        extensionFactoryMap.put("omnetmodel", new XMIResourceFactoryImpl()) //the same for omnet //this is not ned! //.omnetmodel -> model-to-text -> ned, ini, rest

        val dniResource = resourceSet.getResource(dnimm3Uri, true)
        val omnetResource = resourceSet.getResource(omnetUri, true)

		do_transformation(dniResource, omnetResource)		
		
		new XMIResourceImpl => [
			contents.add(omnetResource.contents.get(0))
			save(new FileOutputStream("./output_omnet.xmi"), null)
		]
		
		//omnetResource.save(new FileOutputStream("output_omnet.xml", false), null)
	}
	
	private def static do_transformation(Resource dniResource, Resource omnetResource) {
		
		val dniModel = dniResource.contents.get(0) as NetworkInfrastructure
		val omnetModel = omnetResource.contents.get(0) as NedFile
		
		println("== Starting transformation ==")
		val dni = dniModel
		
		val ned = omnetModel
		val ini = omnetModel.config
		val routingxml = omnetModel.config.routingconf
		//these elements are present in the base file
		
		//routingxml.filename = '''example_«dni.dataCenterName».xml'''
		//or
		routingxml.filename = "example_"+ dni.dataCenterName + ".xml"
		
		
		val netWork = fac.createNetwork
		netWork.name = "Submodules"
		createSimController(netWork)
		val global = fac.createLocalType
		global.name = "GlobalVars"
		val iniGroup = fac.createOmnetppIniGroup
		iniGroup.name = "General"
		
		// how to filter and iterate over all model entities
		for(e: dniResource.allContents.toIterable()) {
 			println(e)
		}
		
		//how to filter and iterate over all model entities of type
		for(e: dniResource.allContents.toIterable.filter(typeof(NamedElement))) {
 			println(e.name)
		}
		
		// how select only some parts of the model starting from the model root (for DNI, model root is NetworkInfrastructure
		val dniCommonNodes = dni.structure.nodes.filter[it.sdntype instanceof Common].map[it as Node]
		for(n : dniCommonNodes){
			parseNodes(netWork, n)
		}
		val CommunicatingApplications = dni.traffic.software.map[it as CommunicatingApplication]
		for(n : CommunicatingApplications){
			parseCommunicatingApp(netWork, iniGroup, n)
		}
		val flows = dni.traffic.flows.map[it as Flow]
		for(n : flows){
			parseFlow(netWork, n)
		}
		//fac.createModuleType
		/*
		////////////////////////////////////////////
		//example Channel
		val chan = fac.createChannel
		chan.name = "Cat6"
		chan.extends = "ned.DatarateChannel"
			val dP = fac.createDoubleParameter
			dP.name = "datarate"
			dP.unit = "Gbps"
			dP.value = 1.0
		chan.parameters.add(dP)
		ned.definitions.add(chan as Channel)
		
		
		////////////////////////////////////////////
		//example two Nodes as SubModules Type Node
		val node1 = fac.createSubModule
		node1.name = "Node1"
		node1.type = "Node"
		val node2 = fac.createSubModule
		node2.name = "Node2"
		node2.type = "Node"


		////////////////////////////////////////////
		//example connections
		val cons = fac.createConnections
		cons.allowunconnected = true
		val con = fac.createChannelConnectionItem
		con.direction = Direction.LEFTTORIGHT;
		 
		con.channelId = "Cat6"
		val portI = fac.createSimplePort
		portI.name = "in"
		portI.increment = true
		val portO = fac.createSimplePort
		portO.name = "out"
		portO.increment = true
		con.leftPort = portI
		con.rightPort = portO
		cons.connections.add(con)
		*/
		
		
		//add to nedFile
		//compMod.submodules.add(node1)
		//compMod.submodules.add(node2)
		//compMod.connections = cons

		ned.definitions.add(global as LocalType)
		ned.definitions.add(netWork as Network)
		ini.sections.add(iniGroup)
		println("== Transformation finished ==")
	}
	
	private def static parseNodes(Network net, Node n)
	{
		val node = fac.createSubModule
		node.name = n.name
		val dniInterfaces = n.interfaces.map[it as NetworkInterface]
		for (iF : dniInterfaces )
		{
			val netIF = fac.createSubModule
			netIF.name = iF.name
			netIF.type = "NetworkInterface"
			net.submodules.add(netIF)
		}
		node.type = "Node"
		net.submodules.add(node)
	}
	
	
	
	private def static parseCommunicatingApp(Network net, OmnetppIniGroup inig,  CommunicatingApplication a)
	{
		val app = fac.createSubModule
		val line = fac.createOmnetppIniLine
		app.name = a.name
		line.nodeSelector = a.name
		line.parameter = "deployedOnNode"
		//val deployed = a.deployedOn.node.name
		if (a.deployedOn!=null )
			line.value = a.deployedOn.node.name
		app.type = "CommunicatingApplication"
		net.submodules.add(app)
		inig.lines.add(line)
	}
	
	private def static parseFlow(Network net, Flow f)
	{
		val dir = fac.createSubModule
		dir.name = f.name
		dir.type = "Flow"
		net.submodules.add(dir)
	}

	
	private def static createSimController(Network net)
	{
		val simCont = fac.createSubModule
		simCont.name = "SimController"
		simCont.type = "SimulationController"
		net.submodules.add(simCont)
	}
	
}
