package tools.descartes.dni.dnimm3ap.tools;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Collections;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import tools.descartes.dni.dnimm3ap.AdaptationPoints;
import tools.descartes.dni.dnimm3ap.DNIAPPackage;
import tools.descartes.dni.dnimm3ap.impl.AdaptationPointsImpl;

public class DNIAPParser {

	private DNIAPParser() {
	}

	public static AdaptationPointsImpl convertToModel(File file) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIAPPackage.eNS_URI, DNIAPPackage.eINSTANCE);
		Resource resource = resourceSet.getResource(URI.createFileURI(file.getAbsolutePath()), true);
		return (AdaptationPointsImpl) resource.getContents().get(0);
	}

	public static AdaptationPointsImpl convertToModel(byte[] data) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIAPPackage.eNS_URI, DNIAPPackage.eINSTANCE);
		Resource resource = resourceSet.createResource(URI.createURI(""));
		resource.load(new ByteArrayInputStream(data), Collections.EMPTY_MAP);
		return (AdaptationPointsImpl) resource.getContents().get(0);
	}

	public static byte[] convertToByteArray(AdaptationPoints adaptationPoints) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIAPPackage.eNS_URI, DNIAPPackage.eINSTANCE);
		Resource resource = resourceSet.createResource(URI.createURI(""));
		resource.getContents().add(adaptationPoints);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		resource.save(baos, Collections.EMPTY_MAP);
		return baos.toByteArray();
	}

	public static void writeToFile(AdaptationPoints adaptationPoints, File file) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIAPPackage.eNS_URI, DNIAPPackage.eINSTANCE);
		Resource resource = resourceSet.createResource(URI.createFileURI(file.getAbsolutePath()));
		resource.getContents().add(DNIAPUtil.cloneModel(adaptationPoints));
		resource.save(Collections.EMPTY_MAP);
	}

}
